@@ -24,17 +24,20 @@
 #include "libs/charmap.h"
 #include "libs/wcontext.h"
 #include "libs/modifiers.h"
+#include "libs/Parse.h"
+#include "libs/Strings.h"
+#include "libs/defaults.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
 #include "functions.h"
 #include "bindings.h"
-#include "defaults.h"
 #include "module_interface.h"
 #include "misc.h"
 #include "screen.h"
 #include "focus.h"
-#include "menus.h"		/* for menu_binding */
+#include "menubindings.h"
+#include "move_resize.h"	/* for placement_binding */
 #ifdef HAVE_STROKE
 #include "stroke.h"
 #endif /* HAVE_STROKE */
@@ -71,7 +74,7 @@ static void update_nr_buttons(
 		return;
 	}
 	/* check for nr_left_buttons */
-	for (i = 0; i < NUMBER_OF_BUTTONS; i += 2)
+	for (i = 0; i < NUMBER_OF_TITLE_BUTTONS; i += 2)
 	{
 		if ((contexts & (C_L1 << i)))
 		{
@@ -82,7 +85,7 @@ static void update_nr_buttons(
 		}
 	}
 	/* check for nr_right_buttons */
-	for (i = 1; i < NUMBER_OF_BUTTONS; i += 2)
+	for (i = 1; i < NUMBER_OF_TITLE_BUTTONS; i += 2)
 	{
 		if ((contexts & (C_L1 << i)))
 		{
@@ -198,7 +201,9 @@ static int bind_get_bound_button_contexts(
 			if (b->Button_Key == 0)
 			{
 				*buttons_grabbed |=
-					((1 << NUMBER_OF_MOUSE_BUTTONS) - 1);
+					((1 <<
+					  NUMBER_OF_EXTENDED_MOUSE_BUTTONS) -
+					 1);
 			}
 			else
 			{
@@ -238,14 +243,15 @@ static int ParseBinding(
 	unsigned short *buttons_grabbed, Bool is_silent)
 {
 	char *action, context_string[20], modifier_string[20], *ptr, *token;
-	char key_string[201] = "";
+	char key_string[201] = "", buffer[80], *windowName = NULL, *p;
 	int button = 0;
 	int n1=0,n2=0,n3=0;
-	KeySym keysym = NoSymbol;
 	int context;
 	int modifier;
-	Bool is_unbind_request = False;
 	int rc;
+	KeySym keysym = NoSymbol;
+	Bool is_unbind_request = False;
+	Bool is_pass_through = False;
 	Bool is_binding_removed = False;
 	Binding *b;
 	Binding *rmlist = NULL;
@@ -254,7 +260,50 @@ static int ParseBinding(
 	STROKE_CODE(int i);
 
 	/* tline points after the key word "Mouse" or "Key" */
-	token = PeekToken(tline, &ptr);
+	token = p = PeekToken(tline, &ptr);
+ 	/* check to see if a window name has been specified. */
+	if (p == NULL)
+	{
+		fvwm_msg(
+			ERR, "ParseBinding", "empty %s binding, ignored\n",tline);
+		return 0;
+	}
+	if (*p == '(')
+	{
+		/* A window name has been specified for the binding. */
+		sscanf(p+1, "%79s",buffer);
+		p = buffer;
+		while (*p != ')')
+		{
+			if (*p == '\0')
+			{
+				if (!is_silent)
+				{
+					fvwm_msg(
+						ERR, "ParseBinding",
+						"Syntax error in line %s -"
+						" missing ')'", tline);
+				}
+				return 0;
+			}
+			++p;
+		}
+		*p++ = '\0';
+		windowName = buffer;
+		if (*p != '\0')
+		{
+			if (!is_silent)
+			{
+				fvwm_msg(
+					ERR, "ParseBinding",
+					"Syntax error in line %s - trailing"
+					" text after specified window", tline);
+			}
+			return 0;
+		}
+		token = PeekToken(ptr, &ptr);
+	}
+
 	if (token != NULL)
 	{
 		if (BIND_IS_KEY_BINDING(type))
@@ -420,16 +469,49 @@ static int ParseBinding(
 	{
 		action++;
 	}
+
+	if (action)
+	{
+		is_pass_through = is_pass_through_action(action);
+		if (is_pass_through)
+		{
+			/* pass-through actions indicate that the event be
+			 * allowed to pass through to the underlying window. */
+			if (windowName == NULL)
+			{
+				/* It doesn't make sense to have a pass-through
+				 * action on global bindings. */
+				if (!is_silent)
+					fvwm_msg(ERR, "ParseBinding",
+						 "Illegal action for global "
+						 "binding: %s", tline);
+				return 0;
+			}
+		}
+	}
 	/* see if it is an unbind request */
-	if (!action || action[0] == '-')
+	if (!action || (action[0] == '-' && !is_pass_through))
 	{
 		is_unbind_request = True;
 	}
 
 	/* short circuit menu bindings for now. */
-	if (context == C_MENU)
+	if ((context & C_MENU) == C_MENU)
 	{
-		return(menu_binding(button,keysym,modifier,action));
+		menu_binding(dpy, type, button, keysym, context,
+				    modifier, action, windowName);
+		/* ParseBinding returns the number of new bindings in pblist
+		 * menu bindings does not add to pblist, and should return 0 */
+		return 0;
+	}
+	/* short circuit placement bindings for now. */
+	if ((context & C_PLACEMENT) == C_PLACEMENT)
+	{
+		placement_binding(button,keysym,modifier,action);
+		/* ParseBinding returns the number of new bindings in pblist
+		 * placement bindings does not add to pblist, and should
+		 * return 0 */
+		return 0;
 	}
 	/*
 	** Remove the "old" bindings if any
@@ -437,7 +519,7 @@ static int ParseBinding(
 	/* BEGIN remove */
 	CollectBindingList(
 		dpy, pblist, &rmlist, type, STROKE_ARG((void *)stroke)
-		button, keysym, modifier, context);
+		button, keysym, modifier, context, windowName);
 	if (rmlist != NULL)
 	{
 		is_binding_removed = True;
@@ -492,7 +574,7 @@ static int ParseBinding(
 		if (button == 0)
 		{
 			*buttons_grabbed |=
-				((1 << NUMBER_OF_MOUSE_BUTTONS) - 1);
+				((1 << NUMBER_OF_EXTENDED_MOUSE_BUTTONS) - 1);
 		}
 		else
 		{
@@ -502,7 +584,7 @@ static int ParseBinding(
 	rc = AddBinding(
 		dpy, pblist, type, STROKE_ARG((void *)stroke)
 		button, keysym, key_string, modifier, context, (void *)action,
-		NULL);
+		NULL, windowName);
 
 	return rc;
 }
@@ -522,7 +604,8 @@ static void binding_cmd(F_CMD_ARGS, binding_t type)
 		Scr.flags.has_mouse_binding_changed = 1;
 		Scr.buttons2grab = btg;
 	}
-	for (b = Scr.AllBindings; count > 0; count--, b = b->NextBinding)
+	for (b = Scr.AllBindings; count > 0 && b != NULL;
+	     count--, b = b->NextBinding)
 	{
 		activate_binding(b, type, True);
 	}
