@@ -54,6 +54,7 @@
 #include "config.h"
 
 #include <stdio.h>
+#include <X11/Xatom.h>
 
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
@@ -336,7 +337,7 @@ ewmh_atom *ewmh_GetEwmhAtomByAtom(Atom atom, ewmh_atom_list_name list_name)
 
 void ewmh_ChangeProperty(
 	Window w, const char *atom_name, ewmh_atom_list_name list,
-	unsigned char *data, unsigned int length)
+	unsigned char *data, int length)
 {
 	ewmh_atom *a;
 	int format = 32;
@@ -381,7 +382,7 @@ static int atom_size(int format)
 }
 
 static
-void *atom_get(Window win, Atom to_get, Atom type, unsigned int *size)
+void *atom_get(Window win, Atom to_get, Atom type, int *size)
 {
 	unsigned char *retval;
 	Atom  type_ret;
@@ -434,7 +435,7 @@ void *atom_get(Window win, Atom to_get, Atom type, unsigned int *size)
 
 void *ewmh_AtomGetByName(
 	Window win, const char *atom_name, ewmh_atom_list_name list,
-	unsigned int *size)
+	int *size)
 {
 	ewmh_atom *a;
 	void *data = NULL;
@@ -725,7 +726,7 @@ void set_kde_sys_tray(void)
 
 void ewmh_AddToKdeSysTray(FvwmWindow *fw)
 {
-	unsigned int size = 0;
+	int size = 0;
 	Atom *val;
 	KstItem *t;
 
@@ -843,7 +844,7 @@ void EWMH_ManageKdeSysTray(Window w, int type)
 
 /**** Client lists ****/
 
-void EWMH_SetClientList()
+void EWMH_SetClientList(void)
 {
 	Window *wl = NULL;
 	FvwmWindow *fw;
@@ -873,7 +874,7 @@ void EWMH_SetClientList()
 	return;
 }
 
-void EWMH_SetClientListStacking()
+void EWMH_SetClientListStacking(void)
 {
 	Window *wl = NULL;
 	FvwmWindow *fw;
@@ -1028,14 +1029,13 @@ void EWMH_UpdateWorkArea(void)
 }
 
 void EWMH_GetWorkAreaIntersection(
-	FvwmWindow *fw, int *x, int *y, unsigned int *w, unsigned int *h,
-	int type)
+	FvwmWindow *fw, int *x, int *y, int *w, int *h, int type)
 {
 	int nx,ny,nw,nh;
 	int area_x = Scr.Desktops->ewmh_working_area.x;
 	int area_y = Scr.Desktops->ewmh_working_area.y;
-	unsigned int area_w = Scr.Desktops->ewmh_working_area.width;
-	unsigned int area_h = Scr.Desktops->ewmh_working_area.height;
+	int area_w = Scr.Desktops->ewmh_working_area.width;
+	int area_h = Scr.Desktops->ewmh_working_area.height;
 	Bool is_dynamic = False;
 
 	switch(type)
@@ -1207,14 +1207,19 @@ Bool ewmh_AllowsYes(EWMH_CMD_ARGS)
 
 Bool ewmh_AllowsClose(EWMH_CMD_ARGS)
 {
-	return is_function_allowed(F_CLOSE, NULL, fw, True, False);
+	return is_function_allowed(
+		F_CLOSE, NULL, fw, RQORIG_PROGRAM_US, False);
 }
 
 Bool ewmh_AllowsFullScreen(EWMH_CMD_ARGS)
 {
-	if (!is_function_allowed(F_MAXIMIZE, NULL, fw, True, False) ||
-	    !is_function_allowed(F_MOVE, NULL, fw, True, False) ||
-	    !is_function_allowed(F_RESIZE, NULL, fw, True, True))
+	if (
+		!is_function_allowed(
+			F_MAXIMIZE, NULL, fw, RQORIG_PROGRAM_US, False) ||
+		!is_function_allowed(
+			F_MOVE, NULL, fw, RQORIG_PROGRAM_US, False) ||
+		!is_function_allowed(
+			F_RESIZE, NULL, fw, RQORIG_PROGRAM_US, True))
 	{
 		return False;
 	}
@@ -1224,22 +1229,22 @@ Bool ewmh_AllowsFullScreen(EWMH_CMD_ARGS)
 
 Bool ewmh_AllowsMinimize(EWMH_CMD_ARGS)
 {
-	return is_function_allowed(F_ICONIFY, NULL, fw, True, False);
+	return is_function_allowed(F_ICONIFY, NULL, fw, RQORIG_PROGRAM_US, False);
 }
 
 Bool ewmh_AllowsMaximize(EWMH_CMD_ARGS)
 {
-	return is_function_allowed(F_MAXIMIZE, NULL, fw, True, False);
+	return is_function_allowed(F_MAXIMIZE, NULL, fw, RQORIG_PROGRAM_US, False);
 }
 
 Bool ewmh_AllowsMove(EWMH_CMD_ARGS)
 {
-	return is_function_allowed(F_MOVE, NULL, fw, True, False);
+	return is_function_allowed(F_MOVE, NULL, fw, RQORIG_PROGRAM_US, False);
 }
 
 Bool ewmh_AllowsResize(EWMH_CMD_ARGS)
 {
-	return is_function_allowed(F_RESIZE, NULL, fw, True, False);
+	return is_function_allowed(F_RESIZE, NULL, fw, RQORIG_PROGRAM_US, False);
 }
 
 void EWMH_SetAllowedActions(FvwmWindow *fw)
@@ -1507,7 +1512,7 @@ void ewmh_HandleWindowType(FvwmWindow *fw, window_style *style)
 {
 	Atom *val;
 	ewmh_atom *list = ewmh_atom_window_type;
-	unsigned int size = 0;
+	int size = 0;
 	int i = 0;
 	Bool found = False;
 
@@ -1599,7 +1604,7 @@ void EWMH_GetStyle(FvwmWindow *fw, window_style *style)
 
 static void ewmh_check_wm_pid(FvwmWindow *fw)
 {
-	unsigned int size = 0;
+	int size = 0;
 	Atom *val;
 
 	fw->ewmh_window_type = 0;
@@ -1787,7 +1792,7 @@ void EWMH_Init(void)
 		Scr.NoFocusWin, "_NET_SUPPORTING_WM_CHECK",
 		EWMH_ATOM_LIST_FVWM_ROOT, (unsigned char *)&val, 1);
 
-	names[0] = "FVWM";
+	names[0] = "fvwm";
 	classhints.res_name= "fvwm";
 	classhints.res_class= "FVWM";
 
@@ -1885,12 +1890,13 @@ void EWMH_fullscreen(FvwmWindow *fw)
 	char cmd[128] = "\0";
 
 	/* maximize with ResizeMoveMaximize */
-	if (!is_function_allowed(
-		    F_MAXIMIZE, NULL, fw, True, False) ||
-	    !is_function_allowed(
-		    F_MOVE, NULL, fw, True, False) ||
-	    !is_function_allowed(
-		    F_RESIZE, NULL, fw, True, True))
+	if (
+		!is_function_allowed(
+			F_MAXIMIZE, NULL, fw, RQORIG_PROGRAM_US, False) ||
+		!is_function_allowed(
+			F_MOVE, NULL, fw, RQORIG_PROGRAM_US, False) ||
+		!is_function_allowed(
+			F_RESIZE, NULL, fw, RQORIG_PROGRAM_US, True))
 	{
 		return;
 	}
@@ -1910,10 +1916,8 @@ void EWMH_fullscreen(FvwmWindow *fw)
 	}
 	SET_EWMH_FULLSCREEN(fw,True);
 	apply_decor_change(fw);
-	fscr.xypos.x =
-		fw->frame_g.x + fw->frame_g.width / 2;
-	fscr.xypos.y =
-		fw->frame_g.y + fw->frame_g.height / 2;
+	fscr.xypos.x = fw->g.frame.x + fw->g.frame.width / 2;
+	fscr.xypos.y = fw->g.frame.y + fw->g.frame.height / 2;
 	FScreenGetScrRect(
 		&fscr, FSCREEN_XYPOS, &scr_g.x, &scr_g.y,
 		&scr_g.width, &scr_g.height);
