@@ -28,6 +28,7 @@
 
 #include <stdio.h>
 #include <signal.h>
+#include <errno.h>
 #include <sys/stat.h>
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
@@ -41,12 +42,20 @@
 #endif
 
 #include <X11/Xproto.h>
+#include <X11/Xatom.h>
 
 #include "libs/fvwmlib.h"
+#include "libs/envvar.h"
+#include "libs/Strings.h"
+#include "libs/System.h"
+#include "libs/Grab.h"
+#include "libs/ColorUtils.h"
+#include "libs/Graphics.h"
 #include "libs/FScreen.h"
 #include "libs/FShape.h"
 #include "libs/PictureBase.h"
 #include "libs/PictureUtils.h"
+#include "libs/Fsvg.h"
 #include "libs/FRenderInit.h"
 #include "libs/charmap.h"
 #include "libs/wcontext.h"
@@ -57,7 +66,7 @@
 #include "misc.h"
 #include "screen.h"
 #include "builtins.h"
-#include "module_interface.h"
+#include "module_list.h"
 #include "colorset.h"
 #include "events.h"
 #include "eventhandler.h"
@@ -66,7 +75,7 @@
 #include "gnome.h"
 #include "ewmh.h"
 #include "add_window.h"
-#include "fvwmsignal.h"
+#include "libs/fvwmsignal.h"
 #include "stack.h"
 #include "virtual.h"
 #include "session.h"
@@ -76,6 +85,7 @@
 #include "move_resize.h"
 #include "frame.h"
 #include "menus.h"
+#include "menubindings.h"
 #include "libs/FGettext.h"
 
 /* ---------------------------- local definitions -------------------------- */
@@ -150,7 +160,8 @@ XContext MenuContext;           /* context for fvwm menus */
 
 int JunkX = 0, JunkY = 0;
 Window JunkRoot, JunkChild;             /* junk window */
-unsigned int JunkWidth, JunkHeight, JunkBW, JunkDepth, JunkMask;
+int JunkWidth, JunkHeight, JunkBW, JunkDepth;
+unsigned int JunkMask;
 
 Bool debugging = False;
 Bool debugging_stack_ring = False;
@@ -164,7 +175,7 @@ char *state_filename = NULL;
 char *restart_state_filename = NULL;  /* $HOME/.fs-restart */
 
 Bool Restarting = False;
-int fd_width, x_fd;
+int x_fd;
 char *display_name = NULL;
 char *fvwm_userdir;
 char const *Fvwm_VersionInfo;
@@ -524,7 +535,7 @@ char *get_display_name(char *display_name, int screen_num)
 /*
  *
  *  Procedure:
- *      Done - tells FVWM to clean up and exit
+ *      Done - tells fvwm to clean up and exit
  *
  */
 /* if restart is true, command must not be NULL... */
@@ -706,7 +717,7 @@ void Done(int restart, char *command)
 	 * SubstructureRedirect selected on the root window ==> windows end up
 	 * in nirvana. This explicitly happened with windows unswallowed by
 	 * FvwmButtons. */
-	ClosePipes();
+	module_kill_all();
 
 	exit(0);
 }
@@ -806,9 +817,9 @@ InstallSignals(void)
 #endif
 #endif
 
-	/* When FVWM restarts, the SIGCHLD handler is automatically reset
+	/* When fvwm restarts, the SIGCHLD handler is automatically reset
 	 * to the default handler. This means that Zombies left over from
-	 * the previous instance of FVWM could still be roaming the process
+	 * the previous instance of fvwm could still be roaming the process
 	 * table if they exited while the default handler was in place.
 	 * We fix this by invoking the SIGCHLD handler NOW, so that they
 	 * may finally rest in peace. */
@@ -1304,6 +1315,9 @@ static void setVersionInfo(void)
 #endif
 #ifdef HAVE_PNG
 	strcat(support_str, " PNG,");
+#endif
+#ifdef HAVE_RSVG
+	strcat(support_str, " SVG,");
 #endif
 	if (FHaveShapeExtension)
 		strcat(support_str, " Shape,");
@@ -1322,6 +1336,9 @@ static void setVersionInfo(void)
 #ifdef HAVE_XRENDER
 	strcat(support_str, " XRender,");
 #endif
+#ifdef HAVE_XCURSOR
+	strcat(support_str, " XCursor,");
+#endif
 #ifdef HAVE_XFT
 	strcat(support_str, " XFT,");
 #endif
@@ -1348,6 +1365,7 @@ static void setVersionInfo(void)
 /* Sets some initial style values & such */
 static void SetRCDefaults(void)
 {
+#define RC_DEFAULTS_COMPLETE ((char *)-1)
 	int i;
 	/* set up default colors, fonts, etc */
 	const char *defaults[][3] = {
@@ -1376,20 +1394,30 @@ static void SetRCDefaults(void)
 		},
 		{
 			"+ \"&4. ",
-			_("Issue FVWM commands"),
+			_("Issue fvwm commands"),
 			"\" Module FvwmConsole"
 		},
 		{
 			"+ \"&R. ",
-			_("Restart FVWM"),
+			_("Restart fvwm"),
 			"\" Restart"
 		},
 		{
 			"+ \"&X. ",
-			_("Exit FVWM"),
+			_("Exit fvwm"),
 			"\" Quit"
 		},
 		{ "Mouse 1 R A Menu MenuFvwmRoot", "", "" },
+		/* default menu navigation */
+		{ "Key Escape M A MenuClose", "", "" },
+		{ "Key Return M A MenuSelectItem", "", "" },
+		{ "Key Left M A MenuCursorLeft", "", "" },
+		{ "Key Right M A MenuCursorRight", "", "" },
+		{ "Key Up M A MenuMoveCursor -1", "", "" },
+		{ "Key Down M A MenuMoveCursor 1", "", "" },
+		{ "Mouse 1 MI A MenuSelectItem", "", "" },
+		/* don't add anything below */
+		{ RC_DEFAULTS_COMPLETE, "", "" },
 		{ "Read "FVWM_DATADIR"/ConfigFvwmDefaults", "", "" },
 		{ NULL, NULL, NULL }
 	};
@@ -1400,6 +1428,11 @@ static void SetRCDefaults(void)
 		exec_context_changes_t ecc;
 		char *cmd;
 
+		if (defaults[i][0] == RC_DEFAULTS_COMPLETE)
+		{
+			menu_bindings_startup_complete();
+			continue;
+		}
 		ecc.type = Restarting ? EXCT_RESTART : EXCT_INIT;
 		ecc.w.wcontext = C_ROOT;
 		exc = exc_create_context(&ecc, ECC_TYPE | ECC_WCONTEXT);
@@ -1408,6 +1441,7 @@ static void SetRCDefaults(void)
 		execute_function(NULL, exc, cmd, 0);
 		exc_destroy_context(exc);
 	}
+#undef RC_DEFAULTS_COMPLETE
 
 	return;
 }
@@ -1427,7 +1461,7 @@ static int CatchFatal(Display *dpy)
 	/* No action is taken because usually this action is caused by someone
 	   using "xlogout" to be able to switch between multiple window managers
 	*/
-	ClosePipes();
+	module_kill_all();
 	exit(1);
 
 	/* to make insure happy */
@@ -1675,6 +1709,34 @@ const char *getInitFunctionName(int n)
 	return initFunctionNames[n >= 0 && n < 3? n: 3];
 }
 
+#ifndef _PATH_DEVNULL
+#	define _PATH_DEVNULL "/dev/null"
+#endif
+static void reopen_fd(int fd, char* mode, FILE *of)
+{
+	struct stat sbuf;
+	FILE *f;
+	int rc;
+
+	errno = 0;
+	rc = fstat(fd, &sbuf);
+	if (rc == 0)
+	{
+		return;
+	}
+	else if (errno != EBADF)
+	{
+		exit(77);
+	}
+	f = freopen(_PATH_DEVNULL, mode, of);
+	if (f == 0 || fileno(f) != fd)
+	{
+		exit(88);
+	}
+
+	return;
+}
+
 /***********************************************************************
  *
  *  Procedure:
@@ -1700,6 +1762,17 @@ int main(int argc, char **argv)
 
 	DBUG("main", "Entered, about to parse args");
 
+	fvwmlib_init_max_fd();
+	/* close open fds */
+	for (i = 3; i < fvwmlib_max_fd; i++)
+	{
+		close(i);
+	}
+	/* reopen stdin, stdout and stderr if necessary */
+	reopen_fd(0, "rb", stdin);
+	reopen_fd(1, "wb", stdout);
+	reopen_fd(2, "wb", stderr);
+
 	memset(&Scr, 0, sizeof(Scr));
 	/* for use on restart */
 	g_argv = (char **)safemalloc((argc + 4) * sizeof(char *));
@@ -1710,8 +1783,8 @@ int main(int argc, char **argv)
 	}
 	g_argv[g_argc] = NULL;
 
-	FlocaleInit(LC_CTYPE, "", "", "FVWM");
-	FGettextInit("fvwm", LOCALEDIR, "FVWM");
+	FlocaleInit(LC_CTYPE, "", "", "fvwm");
+	FGettextInit("fvwm", LOCALEDIR, "fvwm");
 
 	setVersionInfo();
 	/* Put the default module directory into the environment so it can be
@@ -2042,7 +2115,8 @@ int main(int argc, char **argv)
 		if (dn == NULL)
 		{
 			fvwm_msg(
-				ERR, "main", "couldn't find default display (%s)",
+				ERR,
+				"main", "couldn't find default display (%s)",
 				XDisplayName(dn));
 		}
 		else
@@ -2117,7 +2191,6 @@ int main(int argc, char **argv)
 	}
 	FScreenInit(dpy);
 	x_fd = XConnectionNumber(dpy);
-	fd_width = GetFdWidth();
 
 #ifdef HAVE_X11_FD
 	if (fcntl(x_fd, F_SETFD, 1) == -1)
@@ -2320,6 +2393,7 @@ int main(int argc, char **argv)
 	Scr.ColorLimit = PictureInitColors(
 		PICTURE_CALLED_BY_FVWM, True, &colorLimitop, True, True);
 
+	Frsvg_init();
 	FShapeInit(dpy);
 	FRenderInit(dpy);
 	Scr.pscreen = XScreenOfDisplay(dpy, Scr.screen);
@@ -2333,6 +2407,7 @@ int main(int argc, char **argv)
 			XA_CARDINAL, 32, PropModeReplace, NULL, 0);
 
 	Scr.FvwmCursors = CreateCursors(dpy);
+	XDefineCursor(dpy, Scr.Root, Scr.FvwmCursors[CRS_ROOT]);
 	/* create a window which will accept the keyboard focus when no other
 	 * windows have it */
 	/* do this before any RC parsing as some GC's are created from this
@@ -2422,7 +2497,6 @@ int main(int argc, char **argv)
 		Scr.FvwmRoot.cmap_windows = &Scr.NoFocusWin;
 	}
 	InitEventHandlerJumpTable();
-	initModules();
 
 	Scr.gray_bitmap =
 		XCreateBitmapFromData(dpy,Scr.Root,g_bits, g_width,g_height);
@@ -2514,7 +2588,7 @@ int main(int argc, char **argv)
 	valuemask = CWBackPixel | CWColormap | CWBorderPixel;
 
 	Scr.SizeWindow = XCreateWindow(
-		dpy, Scr.Root, 0, 0, 1, 1, (unsigned int)0, Pdepth,
+		dpy, Scr.Root, 0, 0, 1, 1, 0, Pdepth,
 		InputOutput, Pvisual, valuemask, &attributes);
 	resize_geometry_window();
 	initPanFrames();
@@ -2536,7 +2610,7 @@ int main(int argc, char **argv)
 		Done(1, "");       /* does not return */
 
 	default:
-		DBUG("main", "Unknown FVWM run-state");
+		DBUG("main", "Unknown fvwm run-state");
 	}
 
 	exit(0);
