@@ -21,9 +21,11 @@
 #include <stdio.h>
 #include <X11/keysym.h>
 
+#include "libs/Parse.h"
 #include "libs/Bindings.h"
 #include "libs/charmap.h"
 #include "libs/Flocale.h"
+#include "libs/wcontext.h"
 #include "fvwm.h"
 #include "fvwm.h"
 #include "externs.h"
@@ -31,7 +33,6 @@
 #include "misc.h"
 #include "screen.h"
 #include "bindings.h"
-#include "wcontext.h"
 
 #include "move_resize.h"
 #include "menudim.h"
@@ -973,6 +974,17 @@ void menu_shortcuts(
 			newItem = get_selectable_item_from_index(mr, index);
 		}
 
+		if (
+			newItem &&
+			((items_to_move < 0 &&
+			  MI_Y_OFFSET(newItem) > MI_Y_OFFSET(miCurrent)) ||
+			 (items_to_move > 0 &&
+			  MI_Y_OFFSET(newItem) < MI_Y_OFFSET(miCurrent))))
+		{
+			/* never scroll in the "wrong" direction */
+			newItem = NULL;
+		}
+
 		if (newItem)
 		{
 			*pmiCurrent = newItem;
@@ -1007,9 +1019,10 @@ void menu_shortcuts(
 			}
 			else
 			{
+				int old_y = menu_y;
+
 				menu_y += my - menuitem_middle_y_offset(
-					newItem,
-					MR_STYLE(mr));
+					newItem, MR_STYLE(mr));
 
 				if (
 					!MST_SCROLL_OFF_PAGE(mr) &&
@@ -1035,9 +1048,16 @@ void menu_shortcuts(
 							menu_height;
 					}
 				}
-				pmret->rc = MENU_NEWITEM_MOVEMENU;
-				*ret_menu_x = menu_x;
-				*ret_menu_y = menu_y;
+				if (old_y != menu_y)
+				{
+					pmret->rc = MENU_NEWITEM_MOVEMENU;
+					*ret_menu_x = menu_x;
+					*ret_menu_y = menu_y;
+				}
+				else
+				{
+					pmret->rc = MENU_NEWITEM;
+				}
 			}
 		}
 		else
