@@ -31,9 +31,6 @@
 #include "misc.h"
 /* end of for debug message */
 
-/* for fFvwmInStartup */
-#include "externs.h"
-
 /* for get_current_read_file */
 #include "read.h"
 /* for busy cursor */
@@ -42,9 +39,16 @@
 /* for Scr global */
 #include "screen.h"
 
+#include "externs.h"
+
 /* for module syncronous */
 #include "libs/ftime.h"
-#include "fvwmsignal.h"
+#include "libs/System.c"
+#include "libs/envvar.h"
+#include "libs/Parse.h"
+#include "libs/Strings.h"
+#include "libs/wild.h"
+#include "libs/fvwmsignal.h"
 #include "events.h"
 #include "bindings.h"
 
@@ -62,8 +66,6 @@
 
 #define MOD_NOGRABMASK(m) ((m)->xNoGrabMask)
 #define MOD_SYNCMASK(m) ((m)->xSyncMask)
-#define MOD_NEXT(m) ((m)->xnext)
-
 
 typedef struct
 {
@@ -73,18 +75,25 @@ typedef struct
 } mqueue_object_type;
 
 
-
 /* the linked list pointers to the first and last modules */
-static fmodule *module_list = NULL;
-static int num_modules = 0;
+static fmodule_list module_list = NULL;
+/* keep to-be-deleted modules in a deathrow until they are deleted safely. */
+static fmodule_list death_row = NULL;
 
 /*
  * static functions
  */
 static fmodule *module_alloc(void);
 static void module_free(fmodule *module);
-static inline void module_insert(fmodule *module);
-static inline void module_remove(fmodule *module);
+
+/*
+ * list handling functions
+ */
+static inline void module_list_insert(fmodule *module, fmodule_list *list);
+static inline fmodule *module_list_remove(fmodule *module, fmodule_list *list);
+static inline void module_list_destroy(fmodule_list *list);
+static inline int module_list_len(fmodule_list *list);
+
 static void KillModuleByName(char *name, char *alias);
 static char *get_pipe_name(fmodule *module);
 static void DeleteMessageQueueBuff(fmodule *module);
@@ -96,27 +105,14 @@ static void set_message_mask(msg_masks_t *mask, unsigned long msg);
 
 void module_kill_all(void)
 {
-	fmodule *module;
-
-/*
- * this improves speed, but having a single remotion routine should
- * help in mainainability.. replace by module_remove calls?
- */
-	for (module = module_list; module != NULL; module = MOD_NEXT(module))
-	{
-		module_free(module);
-	}
-	module_list = NULL;
-
+	module_list_destroy(&module_list);
 	return;
 }
 
-
 static fmodule *module_alloc(void)
 {
 	fmodule *module;
 
-	num_modules++;
 	module = (fmodule *)safemalloc(sizeof(fmodule));
 	MOD_SET_CMDLINE(module, 0);
 	MOD_READFD(module) = -1;
@@ -127,7 +123,6 @@ static fmodule *module_alloc(void)
 	msg_mask_set(&MOD_SYNCMASK(module), 0, 0);
 	MOD_NAME(module) = NULL;
 	MOD_ALIAS(module) = NULL;
-	MOD_NEXT(module) = NULL;
 
 	return module;
 }
@@ -154,43 +149,52 @@ static void module_free(fmodule *module)
 	{
 		DeleteMessageQueueBuff(module);
 	}
-	num_modules--;
 	free(module);
 
 	return;
 }
 
-static inline void module_insert(fmodule *module)
-{
-	MOD_NEXT(module) = module_list;
-	module_list = module;
-
-	return;
-}
-
-static inline void module_remove(fmodule *module)
+static inline void module_list_insert(fmodule *module, fmodule_list *list)
 {
+	fmodule_store *new_store;
+	DBUG("module_list_insert", "inserting module");
 	if (module == NULL)
 	{
 		return;
 	}
-	if (module == module_list)
+	new_store = (fmodule_store*)safemalloc(sizeof(fmodule_store));
+	new_store->module = module;
+	new_store->next = *list;
+	*list = new_store;
+	return;
+}
+
+static inline fmodule *module_list_remove(fmodule *module, fmodule_list *list)
+{
+	fmodule_store *current;
+
+	if (module == NULL)
 	{
-		DBUG("module_remove", "Removing from module list");
-		module_list = MOD_NEXT(module);
+		return NULL;
+	}
+	if (module == (*list)->module)
+	{
+		DBUG("module_list_remove", "Removing from module list");
+		current=*list;
+		*list = (*list)->next;
+		free(current);
 	}
 	else
 	{
-		fmodule *parent;
-		fmodule *current;
+		fmodule_store *parent;
 
 		/* find it*/
 		for (
-			current = MOD_NEXT(module_list), parent = module_list;
+			current = (*list)->next, parent = (*list);
 			current != NULL;
-			parent = current, current = MOD_NEXT(current))
+			parent = current, current = current->next)
 		{
-			if (current == module)
+			if (current->module == module)
 			{
 				break;
 			}
@@ -198,20 +202,38 @@ static inline void module_remove(fmodule *module)
 		/* remove from the list */
 		if (current != NULL)
 		{
-			DBUG("module_remove", "Removing from module list");
-			MOD_NEXT(parent) = MOD_NEXT(module);
+			DBUG("module_list_remove", "Removing from module list");
+			parent->next = current->next;
+			free(current);
 		}
 		else
 		{
-			fvwm_msg(
-				ERR, "module_remove",
+			DBUG("module_list_remove",
 				"Tried to remove a not listed module!");
-
-			return;
+			return NULL;
 		}
 	}
+
+	return module;
 }
 
+static inline void module_list_destroy(fmodule_list *list)
+{
+	fmodule_store *current;
+	fmodule_store *next;
+
+	for (current = *list; current != NULL; current = next)
+	{
+		next = current->next;
+		module_free(current->module);
+		free(current);
+	}
+	*list = NULL;
+
+	return;
+}
+
+
 
 /*static*/ fmodule *do_execute_module(
 	F_CMD_ARGS, Bool desperate, Bool do_listen_only)
@@ -398,7 +420,7 @@ static inline void module_remove(fmodule *module)
 				"module close-on-exec failed");
 		}
 		/* module struct is completed, insert into the list */
-		module_insert(module);
+		module_list_insert(module, &module_list);
 
 		for (i = 6; i < nargs; i++)
 		{
@@ -461,7 +483,6 @@ static inline void module_remove(fmodule *module)
 			}
 		}
 		free(args);
-		module_remove(module);
 		module_free(module);
 
 		return NULL;
@@ -494,25 +515,32 @@ static inline void module_remove(fmodule *module)
 
 fmodule *executeModuleDesperate(F_CMD_ARGS)
 {
-	return do_execute_module(F_PASS_ARGS, True, False);
+	fmodule *m;
+
+	m = do_execute_module(F_PASS_ARGS, True, False);
+
+	return m;
 }
 
 
 
 void module_kill(fmodule *module)
 {
-	module_remove(module);
-	module_free(module);
-	if (fFvwmInStartup)
-	{
-		/* remove from list of command line modules */
-		DBUG("module_kill", "ending command line module");
-		MOD_IS_CMDLINE(module) = 0;
-	}
+	module_list_insert(
+		module_list_remove(module, &module_list), &death_row);
 
 	return;
 }
 
+/* free modules in the deathrow */
+void module_cleanup(void)
+{
+	module_list_destroy(&death_row);
+
+	return;
+}
+
+
 /* void module_send(fmodule *module, unsigned long *ptr, int size) */
 /* This used to be marked "fvwm_inline".  I removed this
    when I added the lockonsend logic.  The routine seems too big to
@@ -687,14 +715,11 @@ fmodule_input *module_receive(fmodule *module)
 {
 	unsigned long size;
 	unsigned long cont;
+	Window win;
 	int n;
-	fmodule_input *input;
+	fmodule_input *input = NULL;
 
-	input = (fmodule_input *)safemalloc(sizeof(fmodule_input));
-	input->module = module;
-	input->command = NULL;
-
-	n = read(MOD_READFD(module), &(input->window), sizeof(Window));
+	n = read(MOD_READFD(module), &win, sizeof(Window));
 	if (n < sizeof(Window))
 	{
 		/* exit silently, module should have died.. */
@@ -722,8 +747,15 @@ fmodule_input *module_receive(fmodule *module)
 		goto err;
 	}
 
+	/* allocate all storage at once */
 	/* also save space for the '\0' termination character */
-	input->command = (char *)safemalloc(sizeof(char)*size+1);
+	input = (fmodule_input *)safemalloc(
+		sizeof(fmodule_input) + sizeof(char)*(size + 1));
+
+	input->module = module;
+	input->window = win;
+	input->command = (char*)input + sizeof(fmodule_input);
+
 	n = read(MOD_READFD(module), input->command, size);
 	if (n < size)
 	{
@@ -763,15 +795,7 @@ void module_input_discard(fmodule_input *input)
 	{
 		return;
 	}
-
-	if (input->command != NULL)
-	{
-		free(input->command);
-	}
 	free(input);
-
-	/* this avoids problems when freeing the same input multiple times */
-	input=NULL;
 }
 
 /* returns true if the module command matches "expect", false otherwise */
@@ -791,33 +815,50 @@ Bool module_input_expect(fmodule_input *input, char *expect)
 	return False;
 }
 
-fmodule *module_get_next(fmodule *prev)
+void module_list_itr_init(fmodule_list_itr *itr)
 {
-	if (prev == NULL)
-	{
-		return module_list;
-	}
-	else
-	{
-		return MOD_NEXT(prev);
-	}
+	*itr = module_list;
+
+	return;
 }
 
-int module_count(void)
+fmodule *module_list_itr_next(fmodule_list_itr *itr)
 {
-	return num_modules;
+	fmodule *module;
+
+	if (*itr == NULL)
+	{
+		return NULL;
+	}
+	module = (*itr)->module;
+	*itr = (*itr)->next;
+
+	return module;
+}
+
+int module_list_len(fmodule_list *list)
+{
+	int count=0;
+	fmodule_store *current=*list;
+	while (current != NULL)
+	{
+		current = current->next;
+		count++;
+	}
+	return count;
 }
 
 static void KillModuleByName(char *name, char *alias)
 {
+	fmodule_list_itr moditr;
 	fmodule *module;
 
 	if (name == NULL)
 	{
 		return;
 	}
-	module = module_get_next(NULL);
-	for (; module != NULL; module = module_get_next(module))
+	module_list_itr_init(&moditr);
+	while ( (module = module_list_itr_next(&moditr)) != NULL)
 	{
 		if (
 			MOD_NAME(module) != NULL &&
@@ -1029,10 +1070,11 @@ void FlushMessageQueue(fmodule *module)
 
 void FlushAllMessageQueues(void)
 {
+	fmodule_list_itr moditr;
 	fmodule *module;
 
-	module = module_get_next(NULL);
-	for (; module != NULL; module = module_get_next(module))
+	module_list_itr_init(&moditr);
+	while ( (module = module_list_itr_next(&moditr)) != NULL)
 	{
 		FlushMessageQueue(module);
 	}
