@@ -28,9 +28,12 @@
 #endif
 #include <signal.h>
 #include <fcntl.h>
+#include <X11/Xatom.h>
 
-#include "libs/FSMlib.h"
 #include "libs/fvwmlib.h"
+#include "libs/FSMlib.h"
+#include "libs/Strings.h"
+#include "libs/System.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "execcontext.h"
@@ -38,6 +41,7 @@
 #include "misc.h"
 #include "screen.h"
 #include "session.h"
+#include "module_list.h"
 #include "module_interface.h"
 #include "stack.h"
 #include "icccm2.h"
@@ -82,6 +86,8 @@ typedef struct _match
 	int max_x_offset, max_y_offset;
 	int desktop;
 	int layer;
+	int default_layer;
+	int placed_by_button;
 	int used;
 	int gravity;
 	unsigned long ewmh_hint_desktop;
@@ -125,10 +131,10 @@ char *duplicate(const char *s)
 	return r;
 }
 
-static char *get_version_string()
+static char *get_version_string(void)
 {
 	/* migo (14-Mar-2001): it is better to manually update a version string
-	 * in the stable branch, othervise saving sessions becomes useless */
+	 * in the stable branch, otherwise saving sessions becomes useless */
 	return CatString3(VERSION, ", ", __DATE__);
 	/* return "2.6-0"; */
 }
@@ -293,7 +299,7 @@ GetClientID(FvwmWindow *fw)
 		if (actual_type == XA_WINDOW && actual_format == 32 &&
 		    nitems == 1 && bytes_after == 0)
 		{
-			client_leader = *((Window *) prop);
+			client_leader = (Window)(*(long *)prop);
 		}
 	}
 
@@ -305,7 +311,9 @@ GetClientID(FvwmWindow *fw)
 
 	if (client_leader)
 	{
-		if (XGetTextProperty(dpy, client_leader, &tp, _XA_SM_CLIENT_ID))
+		if (
+			XGetTextProperty(
+				dpy, client_leader, &tp, _XA_SM_CLIENT_ID))
 		{
 			if (tp.encoding == XA_STRING && tp.format == 8 &&
 			    tp.nitems != 0)
@@ -398,7 +406,10 @@ SaveWindowStates(FILE *f)
 
 		if (!XGetGeometry(
 			    dpy, FW_W(ewin), &JunkRoot, &JunkX, &JunkY,
-			    &JunkWidth, &JunkHeight, &JunkBW, &JunkDepth))
+			    (unsigned int*)&JunkWidth,
+			    (unsigned int*)&JunkHeight,
+			    (unsigned int*)&JunkBW,
+			    (unsigned int*)&JunkDepth))
 		{
 			/* Don't save the state of windows that already died
 			 * (i.e. modules)! */
@@ -452,6 +463,10 @@ SaveWindowStates(FILE *f)
 			fprintf(f, "  [WINDOW_ROLE] %s\n", window_role);
 			XFree(window_role);
 		}
+		if (client_id && window_role)
+		{
+			/* we have enough information */
+		}
 		else
 		{
 			if (ewin->class.res_class)
@@ -491,7 +506,7 @@ SaveWindowStates(FILE *f)
 
 		gravity_get_naked_geometry(
 			ewin->hints.win_gravity, ewin, &save_g,
-			&ewin->normal_g);
+			&ewin->g.normal);
 		if (IS_STICKY_ACROSS_PAGES(ewin))
 		{
 			save_g.x -= Scr.Vx;
@@ -502,13 +517,13 @@ SaveWindowStates(FILE *f)
 			f, "  [GEOMETRY] %i %i %i %i %i %i %i %i %i %i %i %i"
 			" %i %i %i\n",
 			save_g.x, save_g.y, save_g.width, save_g.height,
-			ewin->max_g.x, ewin->max_g.y, ewin->max_g.width,
-			ewin->max_g.height, ewin->max_g_defect.width,
-			ewin->max_g_defect.height,
+			ewin->g.max.x, ewin->g.max.y, ewin->g.max.width,
+			ewin->g.max.height, ewin->g.max_defect.width,
+			ewin->g.max_defect.height,
 			ig.x + ((!is_icon_sticky_across_pages) ? Scr.Vx : 0),
 			ig.y + ((!is_icon_sticky_across_pages) ? Scr.Vy : 0),
 			ewin->hints.win_gravity,
-			ewin->max_offset.x, ewin->max_offset.y);
+			ewin->g.max_offset.x, ewin->g.max_offset.y);
 		fprintf(f, "  [DESK] %i\n", ewin->Desk);
 		/* set the layer to the default layer if the layer has been
 		 * set by an ewmh hint */
@@ -517,7 +532,8 @@ SaveWindowStates(FILE *f)
 		{
 			layer = Scr.DefaultLayer;
 		}
-		fprintf(f, "  [LAYER] %i\n", layer);
+		fprintf(f, "  [LAYER] %i %i\n", layer, ewin->default_layer);
+		fprintf(f, "  [PLACED_BY_BUTTON] %i\n", ewin->placed_by_button);
 		fprintf(f, "  [EWMH_DESKTOP] %lu\n", ewin->ewmh_hint_desktop);
 		fprintf(f, "  [FLAGS] ");
 		for (i = 0; i < sizeof(window_flags); i++)
@@ -542,7 +558,14 @@ static Bool matchWin(FvwmWindow *w, Match *m)
 	found = 0;
 	client_id = GetClientID(w);
 
-	if (xstreq(client_id, m->client_id))
+	if (Restarting)
+	{
+		if (FW_W(w) == m->win)
+		{
+			found = 1;
+		}
+	}
+	else if (xstreq(client_id, m->client_id))
 	{
 
 		/* client_id's match */
@@ -570,7 +593,6 @@ static Bool matchWin(FvwmWindow *w, Match *m)
 			{
 				/* for non-SM-aware clients we also
 				 * compare WM_COMMAND */
-
 				if (!XGetCommand(
 					    dpy, FW_W(w), &wm_command,
 					    &wm_command_count))
@@ -595,11 +617,10 @@ static Bool matchWin(FvwmWindow *w, Match *m)
 						/* migo (21/Oct/1999):
 						 * on restarts compare
 						 * window ids too */
-						if (!Restarting ||
-						    FW_W(w) == m->win)
-						{
-							found = 1;
-						}
+						/* But if we restart we only need
+						 * to compare window ids
+						 * olicha (2005-01-06) */
+						found = 1;
 					}
 				} /* if (wm_command_count ==... */
 			} /* else if res_class, res_name and wm_name agree */
@@ -1140,8 +1161,7 @@ LoadGlobalState(char *filename)
 			MoveViewport(i1, i2, True);
 		}
 #if 0
-		/* migo (08-Dec-1999): we don't want to eliminate .fvwm2rc for
-		 * now */
+		/* migo (08-Dec-1999): we don't want to eliminate config yet */
 		else if (/*!Restarting*/ 0)
 		{
 			/* Matthias: We don't want to restore too much
@@ -1284,6 +1304,7 @@ LoadWindowStates(char *filename)
 			matches[num_match - 1].icon_y = 0;
 			matches[num_match - 1].desktop = 0;
 			matches[num_match - 1].layer = 0;
+			matches[num_match - 1].default_layer = 0;
 			memset(&(matches[num_match - 1].flags), 0,
 			       sizeof(window_flags));
 			matches[num_match - 1].used = 0;
@@ -1314,9 +1335,15 @@ LoadWindowStates(char *filename)
 			       &(matches[num_match - 1].desktop));
 		}
 		else if (!strcmp(s1, "[LAYER]"))
+		{
+			sscanf(s, "%*s %i %i",
+			       &(matches[num_match - 1].layer),
+			       &(matches[num_match - 1].default_layer));
+		}
+		else if (!strcmp(s1, "[PLACED_BY_BUTTON]"))
 		{
 			sscanf(s, "%*s %i",
-			       &(matches[num_match - 1].layer));
+			       &(matches[num_match - 1].placed_by_button));
 		}
 		else if (!strcmp(s1, "[EWMH_DESKTOP]"))
 		{
@@ -1485,7 +1512,6 @@ MatchWinToSM(
 				}
 			}
 			SET_NAME_CHANGED(ewin,IS_NAME_CHANGED(&(matches[i])));
-			SET_PLACED_WB3(ewin,IS_PLACED_WB3(&(matches[i])));
 			SET_PLACED_BY_FVWM(
 				ewin, IS_PLACED_BY_FVWM(&(matches[i])));
 			ret_state_args->do_shade = IS_SHADED(&(matches[i]));
@@ -1514,19 +1540,19 @@ MatchWinToSM(
 					win_opts->initial_icon_y -= Scr.Vy;
 				}
 			}
-			ewin->normal_g.x = matches[i].x;
-			ewin->normal_g.y = matches[i].y;
-			ewin->normal_g.width = matches[i].w;
-			ewin->normal_g.height = matches[i].h;
-			ewin->max_g.x = matches[i].x_max;
-			ewin->max_g.y = matches[i].y_max;
-			ewin->max_g.width = matches[i].w_max;
-			ewin->max_g.height = matches[i].h_max;
-			ewin->max_g_defect.width = matches[i].width_defect_max;
-			ewin->max_g_defect.height =
+			ewin->g.normal.x = matches[i].x;
+			ewin->g.normal.y = matches[i].y;
+			ewin->g.normal.width = matches[i].w;
+			ewin->g.normal.height = matches[i].h;
+			ewin->g.max.x = matches[i].x_max;
+			ewin->g.max.y = matches[i].y_max;
+			ewin->g.max.width = matches[i].w_max;
+			ewin->g.max.height = matches[i].h_max;
+			ewin->g.max_defect.width = matches[i].width_defect_max;
+			ewin->g.max_defect.height =
 				matches[i].height_defect_max;
-			ewin->max_offset.x = matches[i].max_x_offset;
-			ewin->max_offset.y = matches[i].max_y_offset;
+			ewin->g.max_offset.x = matches[i].max_x_offset;
+			ewin->g.max_offset.y = matches[i].max_y_offset;
 			SET_STICKY_ACROSS_PAGES(
 				ewin, IS_STICKY_ACROSS_PAGES(&(matches[i])));
 			SET_STICKY_ACROSS_DESKS(
@@ -1534,6 +1560,8 @@ MatchWinToSM(
 			ewin->Desk = (IS_STICKY_ACROSS_DESKS(ewin)) ?
 				Scr.CurrentDesk : matches[i].desktop;
 			set_layer(ewin, matches[i].layer);
+			set_default_layer(ewin, matches[i].default_layer);
+			ewin->placed_by_button = matches[i].placed_by_button;
 			/* Note: the Modal, skip pager, skip taskbar and
 			 * "stacking order" state are not restored here: there
 			 * are restored in EWMH_ExitStuff */
@@ -1593,7 +1621,7 @@ RestartInSession (char *filename, Bool isNative, Bool _doPreserveState)
 			"restart us.\n");
 #endif
 		/* Close all my pipes */
-		ClosePipes();
+		module_kill_all();
 
 		exit(0); /* let the SM restart us */
 	}
