@@ -16,6 +16,9 @@
 
 #include "config.h"
 
+#include "libs/fvwmlib.h"
+#include "libs/ColorUtils.h"
+#include "libs/Graphics.h"
 #include "Tools.h"
 
 /* left, center and right offsets */
